#include <sourcemod>
#include <sdktools>
#include <cstrike>
#include <dhooks>
#pragma semicolon 1

public Plugin myinfo =
{
    name = "CS:GO Knives",
    author = "Neuro Toxin",
    description = "A highly refined knife selection plugin",
    version = "1.0.0"
};

int g_iPlayerKnifeSelections[MAXPLAYERS+1];
Handle g_hGiveNamedItem = null;
Handle g_hKnivesMenu = null;

// This data will load via config if this plugin gets anywhere
int g_iKnifeDefinitions[8] = {500,505,506,507,508,509,512,515};
char g_sKnifeDefinitionNames[8][64] = {"Bayonet", "Flip Knife", "Gut Knife", "Karambit", "M9 Bayonet", "Huntsman Knife", "Falchion Knife", "Butterfly Knife" };

public void OnPluginStart()
{
	if (!HookOnGiveNamedItem())
	{
		SetFailState("Unable to hook GiveNamedItem using DHooks");
		return;
	}
	
	BuildKnivesMenu();
	RegisterConsoleCommands();
}

public void RegisterConsoleCommands()
{
	RegConsoleCmd("sm_knife", OnKnifeCommand);
}

public bool HookOnGiveNamedItem()
{
	new Handle:config = LoadGameConfigFile("knives.games");
	if(config == INVALID_HANDLE)
	{
		LogError("Unable to load game config file: knives.games");
		return false;
	}
	
	int offset = GameConfGetOffset(config, "GiveNamedItem");
	CloseHandle(config);
	if (offset == -1)
	{
		LogError("Unable to find offset 'GiveNamedItem' in game data 'knives.games'");
		return false;
	}
	
	g_hGiveNamedItem = DHookCreate(offset, HookType_Entity, ReturnType_CBaseEntity, ThisPointer_CBaseEntity, OnGiveNamedItem);
	
	if (g_hGiveNamedItem == INVALID_HANDLE)
	{
		LogError("Unable to hook 'int CCSPlayer::GiveNamedItem(char const*, int, CEconItemView*, bool)'");
		return false;
	}
	
	DHookAddParam(g_hGiveNamedItem, HookParamType_CharPtr, -1, DHookPass_ByRef);
	DHookAddParam(g_hGiveNamedItem, HookParamType_Int, -1, DHookPass_ByVal);
	DHookAddParam(g_hGiveNamedItem, HookParamType_Int, -1, DHookPass_ByRef);
	DHookAddParam(g_hGiveNamedItem, HookParamType_Bool, -1, DHookPass_ByVal);
	return true;
}

public void OnConfigsExecuted()
{
	// Late loading here is the best place
	for (int client = 1; client < MaxClients; client++)
	{
		if (!IsClientConnected(client))
			continue;
			
		OnClientConnected(client);
		
		if (!IsClientInGame(client))
			continue;
			
		OnClientPutInServer(client);	
	}
}

public OnClientConnected(client)
{
	g_iPlayerKnifeSelections[client] = 0;
}

public OnClientPutInServer(client)
{
	DHookEntity(g_hGiveNamedItem, true, client);
}

public MRESReturn OnGiveNamedItem(int client, Handle hReturn, Handle hParams)
{
	// Skip if player hasnt selected a knife
	if (!g_iPlayerKnifeSelections[client])
		return MRES_Ignored;
	
	// Get classname and skip processing if it's not a weapon
	// -> this is done first as some named items dont contain netprop m_iItemDefinitionIndex
	static char classname[8];
	DHookGetParamString(hParams, 1, classname, sizeof(classname));
	if (!StrEqual(classname, "weapon_"))
		return MRES_Ignored;
	
	// Get entity index and definition index
	int weapon = DHookGetReturn(hReturn);
	int defindex = GetEntProp(weapon, Prop_Send, "m_iItemDefinitionIndex");
	
	// Skip if player is not getting a knife
	bool isknife = IsDefIndexKnife(defindex);
	if (!isknife)
		return MRES_Ignored;
	
	// Set knife definition index to players selection
	SetEntProp(weapon, Prop_Send, "m_iItemDefinitionIndex", g_iPlayerKnifeSelections[client]);
	return MRES_Ignored;
}

stock bool IsDefIndexKnife(int defindex)
{
	if (defindex == 42 || defindex == 59)
		return true;
		
	int count = sizeof(g_iKnifeDefinitions);
	for (int i = 0; i < count; i++)
	{
		if (g_iKnifeDefinitions[i] == defindex)
			return true;
	}
	return false;
}

stock void BuildKnivesMenu()
{
	g_hKnivesMenu = CreateMenu(OnClientMenuAction);
	SetMenuTitle(g_hKnivesMenu, "Select Knife");
	SetMenuExitBackButton(g_hKnivesMenu, false);
	SetMenuExitButton(g_hKnivesMenu, true);
	
	char info[32];
	int count = sizeof(g_iKnifeDefinitions);
	for (int i = 0; i < count; i++)
	{
		Format(info, sizeof(info), "$knives %d", g_iKnifeDefinitions[i]);
		AddMenuItem(g_hKnivesMenu, info, g_sKnifeDefinitionNames[i]);
	}
	
	AddMenuItem(g_hKnivesMenu, "$knives 0", "Default");
}

public Action OnKnifeCommand(int client, int args)
{
	DisplayMenu(g_hKnivesMenu, client, 0);
}

public OnClientMenuAction(Handle menu, MenuAction action, int param1, int param2)
{
	if (action == MenuAction_Select)
	{
		char buffer[64];
		GetMenuItem(menu, param2, buffer, sizeof(buffer));
	
		if (StrContains(buffer, "$knives ") != 0)
			return;
			
		int defindex = StringToInt(buffer[7]);
		if (!GetItemDefinitionName(defindex, buffer, sizeof(buffer)))
			return;
			
		g_iPlayerKnifeSelections[param1] = defindex;
		PrintToChat(param1, "\x01[SM] You have selected knife: \x04%s", buffer);
		
		if (IsPlayerAlive(param1))
		{
			int weapon = GetPlayerWeaponSlot(param1, CS_SLOT_KNIFE);
			if (weapon > -1)
			{
				RemovePlayerItem(param1, weapon);
				GivePlayerItem(param1, "weapon_knife");
			}
		}
	}
}

public bool GetItemDefinitionName(int defindex, char[] buffer, int length)
{
	if (defindex == 0)
	{
		Format(buffer, length, "Default");
		return true;
	}
	
	int count = sizeof(g_iKnifeDefinitions);
	for (int i = 0; i < count; i++)
	{
		if (g_iKnifeDefinitions[i] == defindex)
		{
			Format(buffer, length, "%s", g_sKnifeDefinitionNames[i]);
			return true;
		}
	}
	return false;
}